@@ -4,14 +4,13 @@ use strict;
 use warnings;
 
 use vars qw($VERSION);
-$VERSION = '0.15';
+$VERSION = '0.16';
 
 #--------------------------------------------------------------------------
 
 =head1 NAME
 
-WWW::Scraper::ISBN::AmazonUK_Driver - Search driver for the (UK) Amazon online
-catalog.
+WWW::Scraper::ISBN::AmazonUK_Driver - Search driver for Amazon.co.uk
 
 =head1 SYNOPSIS
 
@@ -26,30 +25,27 @@ Searches for book information from the (UK) Amazon online catalog.
 #--------------------------------------------------------------------------
 
 ###########################################################################
-#Inheritence		                                                      #
-###########################################################################
+# Inheritence
 
 use base qw(WWW::Scraper::ISBN::Driver);
 
 ###########################################################################
-#Library Modules                                                          #
-###########################################################################
+# Modules
 
 use WWW::Mechanize;
-use Template::Extract;
 
 ###########################################################################
-#Constants                                                                #
-###########################################################################
+# Variables
 
-use constant	AMAZON	=> 'http://www.amazon.co.uk/';
-use constant	SEARCH	=> 'http://www.amazon.co.uk/';
+my $AMA_SEARCH = 'http://www.amazon.co.uk/s/ref=nb_sb_noss?url=search-alias=us-stripbooks-tree&field-keywords=%s';
+my $AMA_URL = 'http://www.amazon.co.uk/[^/]+/dp/[\dX]+/ref=sr_1_1.*?sr=1-1';
+my $IN2MM = 25.4;       # number of inches in a millimetre (mm)
+my $OZ2G = 28.3495231;  # number of grams in an ounce (oz)
 
 #--------------------------------------------------------------------------
 
 ###########################################################################
-#Interface Functions                                                      #
-###########################################################################
+# Public Interface
 
 =head1 METHODS
 
@@ -72,6 +68,11 @@ a valid page is returned, the following fields are returned via the book hash:
   image_link
   pubdate
   publisher
+  binding   (if known)
+  pages     (if known)
+  weight    (if known) (in grammes)
+  width     (if known) (in millimetres)
+  height    (if known) (in millimetres)
 
 The book_link, thumb_link and image_link refer back to the Amazon (UK) website.
 
@@ -87,50 +88,56 @@ sub search {
 
 	my $mechanize = WWW::Mechanize->new();
     $mechanize->agent_alias( 'Linux Mozilla' );
-	$mechanize->get( SEARCH );
+
+    my $search = sprintf $AMA_SEARCH, $isbn;
+
+    $mechanize->get( $search );
     return $self->handler("Amazon UK website appears to be unavailable.")
 	    unless($mechanize->success());
 
 	my $content = $mechanize->content();
-#print STDERR "\n# content1=[".$mechanize->content()."]\n";
-
-	my ($keyword) = ($content =~ /<option value="(.*?stripbooks.*?)">Books/);
-	$mechanize->form_name('site-search');
-	$mechanize->set_fields( 'field-keywords' => $isbn, 'url' => 'search-alias=stripbooks' );
-	$mechanize->submit();
+    my ($link) = $content =~ m!($AMA_URL)!s;
+	return $self->handler("Failed to find that book on Amazon UK website.")
+	    unless($link);
 
+    $mechanize->get( $link );
 	return $self->handler("Failed to find that book on Amazon UK website.")
 	    unless($mechanize->success());
 
 	# The Book page
-	my $template = <<END;
-<title>[% content %]</title>[% ... %]
-registerImage("original_image", "[% thumb_link %]"[% ... %]
-<a href="+'"'+"[% image_link %]"[% ... %]
-Product details[% ... %]
-<b>Publisher:</b> [% publisher %] ([% pubdate %])[% ... %]
-<li><b>ISBN-10:</b> [% isbn10 %]</li>[% ... %]
-<li><b>ISBN-13:</b> [% isbn13 %]</li>[% ... %]
-END
+    my $html = $mechanize->content;
+    my $data = {};
 
-#print STDERR "\n# content2=[".$mechanize->content()."]\n";
+    #print STDERR "\n#$html";
 
-    my $extract = Template::Extract->new;
-    my $data = $extract->extract($template, $mechanize->content());
-
-	return $self->handler("Could not extract data from Amazon UK result page.")
-		unless(defined $data);
+    ($data->{binding},$data->{pages})   = $html =~ m!<li><b>(Paperback|Hardcover):</b>\s*([\d.]+)\s*pages</li>!s;
+    ($data->{weight})                   = $html =~ m!<li><b>Shipping Weight:</b>\s*([\d.]+)\s*ounces</li>!s;
+    ($data->{height},$data->{width})    = $html =~ m!<li><b>\s*Product Dimensions:\s*</b>\s*([\d.]+) x ([\d.]+) x ([\d.]+) cm\s*</li>!s;
+    ($data->{published})                = $html =~ m!<li><b>Publisher:</b>\s*(.*?)</li>!s;
+    ($data->{isbn10})                   = $html =~ m!<li><b>ISBN-10:</b>\s*(.*?)</li>!s;
+    ($data->{isbn13})                   = $html =~ m!<li><b>ISBN-13:</b>\s*(.*?)</li>!s;
+    ($data->{content})                  = $html =~ m!<meta name="description" content="([^"]+)"!s;
 
     $data->{content} =~ s/Amazon\.co\.uk.*?://i;
     $data->{content} =~ s/: Books.*//i;
 	($data->{title},$data->{author}) = ($data->{content} =~ /\s*(.*?)(?:\s+by|,|:)\s+([^:]+)\s*$/)  unless($data->{author});
 
-    # trim top and tail
-	foreach (keys %$data) { $data->{$_} =~ s/^\s+//;$data->{$_} =~ s/\s+$//; }
+	($data->{thumb_link},$data->{image_link})  
+                                        = $html =~ m!registerImage\("original_image",\s*"([^"]+)",\s*"<a href="\+'"'\+"([^"]+)"\+!;
+
+    ($data->{publisher},$data->{pubdate}) = ($data->{published} =~ /\s*(.*?)(?:;.*?)?\s+\((.*?)\)/);
+    $data->{isbn10}  =~ s/[^\dX]+//g;
+    $data->{isbn13}  =~ s/\D+//g;
 	$data->{pubdate} =~ s/^.*?\(//;
-    $data->{isbn13} =~ s/-//g;
+
+    $data->{width}  = int($data->{width} * 10)  if($data->{width});
+    $data->{height} = int($data->{height} * 10) if($data->{height});
+
+    # trim top and tail
+	foreach (keys %$data) { next unless(defined $data->{$_});$data->{$_} =~ s/^\s+//;$data->{$_} =~ s/\s+$//; }
 
 	my $bk = {
+		'ean13'		    => $data->{isbn13},
 		'isbn13'		=> $data->{isbn13},
 		'isbn10'		=> $data->{isbn10},
 		'isbn'			=> $data->{isbn10},
@@ -140,7 +147,13 @@ END
 		'thumb_link'	=> $data->{thumb_link},
 		'publisher'		=> $data->{publisher},
 		'pubdate'		=> $data->{pubdate},
-		'book_link'		=> $mechanize->uri()
+		'book_link'		=> $mechanize->uri(),
+		'content'		=> $data->{content},
+		'binding'	    => $data->{binding},
+		'pages'		    => $data->{pages},
+		'weight'		=> $data->{weight},
+		'width'		    => $data->{width},
+		'height'		=> $data->{height}
 	};
 	$self->book($bk);
 	$self->found(1);
@@ -164,6 +177,18 @@ L<WWW::Scraper::ISBN>,
 L<WWW::Scraper::ISBN::Record>,
 L<WWW::Scraper::ISBN::Driver>
 
+=head1 BUGS, PATCHES & FIXES
+
+There are no known bugs at the time of this release. However, if you spot a
+bug or are experiencing difficulties that are not explained within the POD
+documentation, please send an email to barbie@cpan.org or submit a bug to the
+RT system (http://rt.cpan.org/Public/Dist/Display.html?Name=WWW-Scraper-ISBN-Amazon_Driver).
+However, it would help greatly if you are able to pinpoint problems or even
+supply a patch.
+
+Fixes are dependant upon their severity and my availablity. Should a fix not
+be forthcoming, please feel free to (politely) remind me.
+
 =head1 AUTHOR
 
   Barbie, <barbie@cpan.org>
@@ -171,13 +196,9 @@ L<WWW::Scraper::ISBN::Driver>
 
 =head1 COPYRIGHT & LICENSE
 
-  Copyright (C) 2004-2007 Barbie for Miss Barbell Productions
+  Copyright (C) 2004-2010 Barbie for Miss Barbell Productions
 
   This module is free software; you can redistribute it and/or
-  modify it under the same terms as Perl itself.
-
-The full text of the licenses can be found in the F<Artistic> file included
-with this module, or in L<perlartistic> as part of Perl installation, in
-the 5.8.1 release or later.
+  modify it under the Artistic Licence v2.
 
 =cut
